import React, { useRef, useState } from "react";
import throttle from "lodash.throttle";
import usePaintSquare from "./usePaintSquare";
import { usePicker } from "./context";

const Square = () => {
  const { handleColor, x, y, internalHue } = usePicker();
  const [dragging, setDragging] = useState(false);
  const canvas = useRef(null);
  usePaintSquare(canvas, internalHue);

  const handleChange = e => {
    const ctx = canvas?.current?.getContext("2d");
    const onMouseMove = throttle(() => handleColor(e, ctx), 250);
    onMouseMove();
  };

  const stopDragging = () => {
    setDragging(false);
  };

  const handleMove = e => {
    if (dragging) {
      handleChange(e);
    }
  };

  const handleClick = e => {
    if (!dragging) {
      handleChange(e);
    }
  };

  return (
    <div style={{ position: "relative" }}>
      <div
        style={{
          position: "absolute",
          left: -7,
          top: -7,
          width: 308,
          height: 308
        }}
        onMouseEnter={stopDragging}
      />
      <div
        style={{ position: "relative", cursor: "crosshair" }}
        onMouseMove={e => handleMove(e)}
        onMouseUp={stopDragging}
      >
        <div
          style={{
            left: x,
            top: y,
            position: "absolute",
            border: "2px solid white",
            borderRadius: "50%",
            boxShadow: "0px 0px 3px rgba(0,0,0,.5)",
            width: 18,
            height: 18,
            zIndex: 1000
          }}
          className="handle"
          onMouseDown={() => setDragging(true)}
        />
        <div className="canvas-wrapper" onClick={e => handleClick(e)}>
          <canvas ref={canvas} width="294px" height="294px" id="paintSquare" />
        </div>
      </div>
    </div>
  );
};

export default Square;
